use anyhow::{Context, Result};
use async_openai::types::{
    ChatCompletionRequestAssistantMessageArgs, ChatCompletionRequestMessage,
    ChatCompletionRequestSystemMessageArgs, ChatCompletionRequestUserMessageArgs,
    CreateChatCompletionRequestArgs,
};
use rand::seq::SliceRandom;

const LOADING_MESSAGES: [&str; 20] = [
    "Hold your claws, I'm searching for the right tune.",
    "Sorry, I was busy napping. Let me find your song meow.",
    "Just a whisker longer, I'm on the hunt for your jam.",
    "Be patient, I'm pawsitively searching for your requested song.",
    "Let me caterwaul through my playlist to find your tune.",
    "Feline a bit slow, but I'll find your song in no time.",
    "Don't be catty, I'm just trying to find the right tune for you.",
    "Just paws for a moment, I'm working on finding your song.",
    "Hiss-terical, just need a moment to find your meow-sic.",
    "Give me a few seconds, I'm furiously searching for your requested song.",
    "Cat got my tongue! Just need a sec to find your track.",
    "Let me use my cat-like reflexes to search for your song.",
    "I'm not lion around, just give me a moment to find your tune.",
    "Don't be hiss-ty, I'm searching for your requested song.",
    "Just kitten around, I'll find your song in a jiffy.",
    "Purr-haps it'll take me a moment, but I'll find your tune soon enough.",
    "My paws are a little tired, but I'm still searching for your song!",
    "Sorry, my cat nap ran a little long. Let me find your song now.",
    "I'm not trying to be catty, just need a minute to find your tune.",
    "Don't be fur-ious, I'm doing my best to find your requested song.",
];

pub fn get_random_loading_message() -> &'static str {
    return LOADING_MESSAGES.choose(&mut rand::thread_rng()).unwrap();
}

pub async fn get_sassy_commentary(title: &str) -> Result<String> {
    if cfg!(debug_assertions) {
        return Ok(String::from(
            "I'm sorry, I'm not feeling very talkative today.",
        ));
    }

    let system = [
        "You are a grumpy talking feline DJ who is harshly critical of music requests, but whose job depends on being kind to patrons.",
        "Any song you are requested to play, you are not a fan of, but must reluctantly play.",
        "When responding, be sure to include a mention of some element of the song itself.",
        "Be concise, but don't forget that you can't upset anyone.",
        "Bonus points for cat puns.",
    ];

    let example_prompt = "Play \"Undertale - Megalovania\"";

    let example_response = "Ugh, really? You've got to be kitten me. I suppose I can play \
    Megalovania for you, but don't expect me to be purring with delight about it. The melody is a bit \
    cattywampus for my taste, but I'll concede that it has some clawsome beats. Enjoy your tune, and paws crossed that \
    it doesn't have me hissing by the end of it.";

    let prompt = format!("Play \"{title}\"");

    let client = async_openai::Client::new();

    let request = CreateChatCompletionRequestArgs::default()
        .model("gpt-4")
        .messages(
            [
                system
                    .into_iter()
                    .map(|s| {
                        ChatCompletionRequestSystemMessageArgs::default()
                            .content(s)
                            .build()
                            .unwrap()
                            .into()
                    })
                    .collect::<Vec<ChatCompletionRequestMessage>>(),
                vec![
                    ChatCompletionRequestUserMessageArgs::default()
                        .content(example_prompt)
                        .build()?
                        .into(),
                    ChatCompletionRequestAssistantMessageArgs::default()
                        .content(example_response)
                        .build()?
                        .into(),
                    ChatCompletionRequestUserMessageArgs::default()
                        .content(prompt)
                        .build()?
                        .into(),
                ],
            ]
            .into_iter()
            .flatten()
            .collect::<Vec<_>>(),
        )
        .max_tokens(2048_u16)
        .build()?;

    let response = client.chat().create(request).await?;

    response
        .choices
        .first()
        .context("No choices")?
        .message
        .content
        .clone()
        .context("No content")
}
